Ext.namespace('SYNO.SDS');

var _S = function(key) {
  return SYNO.SDS.Session[key] || '';
};

var _TranslateError = function(err) {
  var default_msg = _T('login', 'forget_pass_msg_ask_admin');
  try {
    return _T(err.i18n.split(':')[0], err.i18n.split(':')[1]) || err.text;
  } catch(e1) {
    try {
      return err.text || default_msg;
    } catch(e2) {
      return default_msg;
    }
  }
};

Ext.define('SYNO.SDS.ADServer.TicketForm', {
  extend: 'SYNO.SDS.SimpleForm',
  constructor: function(override) {
    var config = Ext.apply({
      cls: 'change-user-pass-page',
      title: _T('forgot_pass', 'forgot_form_title'),
      desc: _T('forgot_pass', 'forgot_form_desc'),
      btnText: _T('forgot_pass', 'send_mail')
    }, override);
    this.callParent([config]);
  },
  getInputFields: function() {
    this.nameField = new SYNO.SDS.IconTextfield({
      fieldLabel: _T('common', 'username'),
      emptyText: _T('common', 'username'),
      allowBlank: false,
      cls: 'user-field'
    });
    return [this.nameField];
  },
  initFields: function() {
    if (Ext.isObject(_S('err'))) {
      this.setError(_TranslateError(_S('err')));
    }
    if (_S('username')) {
      this.nameField.setValue(_S('username'));
    }
    this.nameField.focus();
  },
  onSubmit: function() {
    var username = this.nameField.getValue();
    this.nameField[username ? 'removeClass' : 'addClass']('error');
    if (!username) {
      this.setError(_T('login', 'forget_pass_enter_user_name'));
      return false;
    }

    this.clearMsg();
    this.sendBtn.disable();
    this.setMsg(_T('common','msg_waiting'));
    Ext.Ajax.request({
      url: './reset_password.cgi',
      'Content-type': "application/x-www-form-urlencoded",
      method: 'POST',
      params: {
        user: username,
        expect: 'ajax'
      },
      scope: this,
      success: function(resp, options) {
        var data = Ext.decode(resp.responseText);
        this.clearMsg();
        this.sendBtn.enable();
        if (data.err && data.err.success) {
          this.fireEvent('action_finished');
        } else {
          this.setError(_TranslateError(data.err));
        }
      }
    });
  }
});

Ext.define('SYNO.SDS.ADServer.PasswordForm', {
  extend: 'SYNO.SDS.SimpleForm',
  constructor: function(override) {
    var config = Ext.apply({
      cls: 'change-pass-form',
      title: _T('forgot_pass', 'change_pass_title'),
      desc: _T('forgot_pass', 'change_pass_desc'),
      btnText: _T('common', 'submit')
    }, override);
    this.callParent([config]);
  },
  getInputFields: function() {
    this.passField = new SYNO.SDS.IconTextfield({
      cls: 'passwd-field',
      width: 320,
      inputType: 'password',
      fieldLabel: _T("user", "user_passwd_new"),
      emptyText: _T("user", "user_passwd_new")
    });
    this.confirmField = new SYNO.SDS.IconTextfield({
      cls: 'confirm-field',
      width: 320,
      inputType: 'password',
      fieldLabel: _T("user", "user_repswd"),
      emptyText: _T("user", "user_repswd"),
    });
    return [this.passField, this.confirmField];
  },
  initFields: function() {
    this.passField.focus();
  },
  onSubmit: function() {
    var password = this.passField.getValue();
    this.passField[password ? 'removeClass' : 'addClass']('error');
    if (!password) {
      this.setError(_T('login', 'forget_pass_enter_password'));
      return false;
    }

    var is_valid = (password === this.confirmField.getValue());
    this.confirmField[is_valid ? 'removeClass' : 'addClass']('error');
    if (!is_valid) {
      this.setError(_T('login', 'forget_pass_comfirm_password_error'));
      return false;
    }

    this.clearMsg();
    this.sendBtn.disable();
    this.setMsg(_T('common','msg_waiting'));
    Ext.Ajax.request({
      url: '/webapi/encryption.cgi',
      method: 'POST',
      params: {
        api: 'SYNO.API.Encryption',
        method: 'getInfo',
        version: 1,
        format: 'module'
      },
      scope: this,
      success: function(resp, options) {
        var obj = Ext.decode(resp.responseText);
        if (obj.success) {
          SYNO.Encryption.CipherKey = obj.data.cipherkey;
          SYNO.Encryption.RSAModulus = obj.data.public_key;
          SYNO.Encryption.CipherToken = obj.data.ciphertoken;
          SYNO.Encryption.TimeBias = obj.data.server_time - Math.floor(+new Date()/1000);
        }
        var encrypted_param = SYNO.Encryption.EncryptParam({
          ticket: _S('ticket'),
          password: password
        });
        this.resetPassword({'__cIpHeRtExT': encrypted_param[obj.data.cipherkey] || ''});
      }
    });
  },
  resetPassword: function(params) {
    Ext.Ajax.request({
      url: './reset_password.cgi',
      'Content-type': "application/x-www-form-urlencoded",
      method: 'POST',
      params: params,
      scope: this,
      success: function(resp, options) {
        var data = Ext.decode(resp.responseText);
        this.clearMsg();
        this.sendBtn.enable();
        if (data.err && data.err.success) {
          this.fireEvent('action_finished');
        } else if (data.err && data.err.weak_password) {
          var reasons = [
            _T('passwd', 'mixed_case'),
            _T('passwd', 'included_special_char'),
            _T('passwd', 'included_numeric_char'),
            _T('passwd', 'need_longer_password'),
            _T('passwd', 'exclude_username')
          ];
          this.setError(String.format(_T('passwd', 'possible_reasons_are'), reasons.join(', ')));
        } else {
          this.setError(_TranslateError(data.err));
        }
      }
    });
  }
});


Ext.define('SYNO.SDS.ADServer.EndingForm', {
  extend: 'SYNO.ux.Panel',
  constructor: function(override) {
    var config = {
      cls: 'sds-simple-form',
      hidden: true,
      items: [{
        xtype: 'syno_displayfield',
        cls: 'form-title',
        value: override.title
      }, {
        xtype: 'syno_displayfield',
        cls: 'form-desc',
        value: override.desc
      }, {
        hidden: _S('ticket'),
        xtype: 'box',
        cls: 'ticket-sent-icon'
      }, {
        xtype: 'syno_button',
        text: _T('forgot_pass', 'login_now'),
        hidden: !_S('redirect_url'),
        width: 194,
        height: 40,
        scope: this,
        handler: function() {
          window.location = _S('redirect_url');
        }
      }],
      listeners: {
        afterlayout: function(me) {
          me.el.alignTo(Ext.getBody(), 'c-c');
        },
        scope: this
      }
    };
    this.callParent([config]);
  }
});


Ext.define('SYNO.SDS.ADServer.ResetPasswordPage', {
  extend: 'Ext.Container',
  constructor: function(override) {
    if (_S('ticket')) {
      this.actionForm = new SYNO.SDS.ADServer.PasswordForm({owner:this});
      this.endingForm = new SYNO.SDS.ADServer.EndingForm({
        title: _T('forgot_pass', 'change_success_title'),
        desc: _T('forgot_pass', 'change_success_desc')
      });
    } else {
      this.actionForm = new SYNO.SDS.ADServer.TicketForm({owner:this});
      this.endingForm = new SYNO.SDS.ADServer.EndingForm({
        title: _T('forgot_pass', 'check_mail_title'),
        desc: _T('forgot_pass', 'check_mail_desc')
      });
    }
    var config = Ext.apply({
      cls: 'change-pass-page forgot-pass-background',
      renderTo: document.body,
      items: [this.actionForm, this.endingForm],
      listeners: {
        afterrender: function() {
          this.actionForm.initFields();
        },
        show: function() {
          if (_S('err').success) {
            this.actionForm.fireEvent('action_finished');
          }
        },
        scope: this
      }
    }, override);
    this.callParent([config]);
    this.mon(this.actionForm, 'action_finished', this.onFinished, this);
  },
  destroy: function() {
    this.actionForm.destroy();
    this.endingForm.destroy();
    this.callParent();
  },
  onFinished: function() {
    this.actionForm.hide();
    this.endingForm.show();
  }
});


Ext.onReady(function() {
  Ext.getBody().setStyle('overflow', 'hidden');
  var page = new SYNO.SDS.ADServer.ResetPasswordPage();
  page.show();
});

