#! /bin/bash
source /var/packages/DirectoryServerForWindowsDomain/scripts/pkg_utils

# Samba ask for 600 permission for private key
# Samba explains that this requirement is related to CVE-2013-4476
#       (shows in /var/log/samba/log.samba if permission of privkey is not 600)
# However, the original fix requirement is to ask for go-rwx
#   and nobody knows why samba need u+w in addition.

if [[ ! -s "${SMB_CERT_PATH}/privkey.pem" || ! -s "${SYNOADS_CONF_HASH}" ]]; then
    exit 0
fi

chmod 600 "${SMB_CERT_PATH}/privkey.pem"

REALM=$(jq -r '.realm' "${SYNOADS_CONF_HASH}")
HOSTNAME=$(jq -r '.hostname' "${SYNOADS_CONF_HASH}")
DC_FQDN="${HOSTNAME}.${REALM}"
CERT_FQDN=$(openssl x509 -in "${SMB_CERT_PATH}/cert.pem" \
    -subject -noout 2>/dev/null \
    | sed -e 's/^subject.*CN=\([a-zA-Z0-9.*_-]*\).*$/\1/')

if [[ -s "${SMB_CERT_PATH}/syno-ca-cert.pem" ]]; then
    CA_FILE="${SMB_CERT_PATH}/syno-ca-cert.pem"
fi

echo "dc fqdn: ${DC_FQDN}"
echo "cert fqdn: ${CERT_FQDN}"

# do not double quote the right-hand side of equation.
# it should be true if dc = xx.example.com and certificate is registered on *.example.com.
if [[ "${DC_FQDN}" = ${CERT_FQDN,,} ]]; then
    cat <<JSON > "${SYNOPKG_PKGDEST}/resource/certificate/tls.json"
{
    "support-tls": true,
    "tls-keyfile": "${SMB_CERT_PATH}/privkey.pem",
    "tls-certfile": "${SMB_CERT_PATH}/cert.pem",
    "tls-cafile": "${CA_FILE}"
}
JSON
else
    echo "{}" > "${SYNOPKG_PKGDEST}/resource/certificate/tls.json"
fi

logger -p user.err "Certificate of Directory Server For Windows Domain is changed"

if pidof samba > /dev/null; then
    restart pkg-"${SYNOPKG_PKGNAME}"
fi
